#--------------------------------------------------------------------
#----- GRNOC TL1 ADVA FSP3000 Device Module
#-----
#----- Copyright(C) 2013 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: svn+ssh://svn.grnoc.iu.edu/grnoc/perl-lib/GRNOC-TL1/tags/1.3.10/lib/GRNOC/TL1/Device/ADVA/FSP3000.pm $
#----- $Id: FSP3000.pm 22521 2013-02-05 20:23:28Z mrmccrac $
#-----
#----- This module handles the functionality for the ADVA FSP3000
#----- device.  It inherts from the standard GRNOC::TL1::Device
#----- module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::ADVA::FSP3000;

use strict;
use warnings;

use base "GRNOC::TL1::Device";

use Date::Calc qw(Mktime);
use Data::Dumper;

sub new {

    my $caller = shift;
    my $class = ref($caller) || $caller;

    my (%args) = @_;

    # don't wait for login prompt by default
    $args{'wait_for_login_prompt'} = 0 if ($args{'wait_for_login_prompt'} eq '');
    $args{'port'} = 2024 if (! $args{'port'});

    # use ; as default prompt
    $args{'prompt'} = ';' if ($args{'prompt'} eq '');

    # call the GRNOC::TL1::Device structure
    my $self = $class->SUPER::new(%args);

    bless($self, $class);

    return $self;
}

sub get_optical_interfaces {

    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-OL:$tid" . ":ALL:$ctag;";

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined($cmd_output));

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    my @interfaces;

    foreach my $response (@responses) {

        my $interface;

        if ($response =~ /^"([^:]+)::.*:([^"]*)"$/) {

            $interface->{'aid'} = $1;
            $interface->{'state'} = $2;

            ($interface->{'mode'}) = $response =~ /AMPMODE=([^,:]*)/;
            ($interface->{'rx_loss'}) = $response =~ /INPUTLOSS=([^,:]*)/;
            ($interface->{'tx_loss'}) = $response =~ /OUTPUTLOSS=([^,:]*)/;
            ($interface->{'gain'}) = $response =~ /[,:]GAIN=([^,:]*)/;
            ($interface->{'description'}) = $response =~ /ALIAS=\\"([^\\"]*)\\",/;

            push(@interfaces, $interface);
        }

        else {

            print STDERR "get_optical_interfaces: Error parsing response: $response\n";
        }
    }

    return @interfaces;
}

sub get_optical_channels {

    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-CH:$tid" . ":ALL:$ctag;";

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined($cmd_output));

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    my @channels;

    foreach my $response (@responses) {

        my $channel;

        #"CH-1-5-N:OTU2LAN:ALIAS=\"\",ALSMODE=ALS,ERRFWD=AIS,LSROFFDLY=DISABLE,MON=OPU,VSTIMER=08-00,PATHID=NA,STUFF=Y,FEC=GFEC,TCMA=DSBLD,TCMB=DSBLD,TCMC=DSBLD,TERM=OPU,SDHT-OTU=15,SDPER-OTU=7,SDHT-ODU=15,SDPER-ODU=7,SDHT-TCMA=15,SDPER-TCMA=7,SDHT-TCMB=15,SDPER-TCMB=7,SDHT-TCMC=15,SDPER-TCMC=7,CHANNEL=19410,FREQNO=194.10,LAMBDA=1544.53,REACH=VLONG,BITRATE=11095,TRC-EXP-SAPI-ODU=\"\",TRC-EXP-SAPI-OTU=\"\",TRC-EXP-SAPI-TCMA=\"\",TRC-EXP-SAPI-TCMB=\"\",TRC-EXP-SAPI-TCMC=\"\",TRC-RCV-DAPI-ODU=\"\",TRC-RCV-DAPI-OTU=\"\",TRC-RCV-DAPI-TCMA=\"\",TRC-RCV-DAPI-TCMB=\"\",TRC-RCV-DAPI-TCMC=\"\",TRC-RCV-OPSP-ODU=\"\",TRC-RCV-OPSP-OTU=\"\",TRC-RCV-OPSP-TCMA=\"\",TRC-RCV-OPSP-TCMB=\"\",TRC-RCV-OPSP-TCMC=\"\",TRC-RCV-SAPI-ODU=\"\",TRC-RCV-SAPI-OTU=\"\",TRC-RCV-SAPI-TCMA=\"\",TRC-RCV-SAPI-TCMB=\"\",TRC-RCV-SAPI-TCMC=\"\",TRC-TRMT-DAPI-ODU=\"\",TRC-TRMT-DAPI-OTU=\"\",TRC-TRMT-DAPI-TCMA=\"\",TRC-TRMT-DAPI-TCMB=\"\",TRC-TRMT-DAPI-TCMC=\"\",TRC-TRMT-OPSP-ODU=\"\",TRC-TRMT-OPSP-OTU=\"\",TRC-TRMT-OPSP-TCMA=\"\",TRC-TRMT-OPSP-TCMB=\"\",TRC-TRMT-OPSP-TCMC=\"\",TRC-TRMT-SAPI-ODU=\"\",TRC-TRMT-SAPI-OTU=\"\",TRC-TRMT-SAPI-TCMA=\"\",TRC-TRMT-SAPI-TCMB=\"\",TRC-TRMT-SAPI-TCMC=\"\",TIMMODE-OTU=DSBLD,TIMMODE-TCMA=DSBLD,TIMMODE-TCMB=DSBLD,TIMMODE-TCMC=DSBLD,TIMMODE-ODU=DSBLD,TIMING=LOOP,BEHAVE=NETWORK,LFTIMER=00-00:OOS-AU,AINS&LKDO&FAF"
        if ($response =~ /^\"([^:]+):([^:]*):.*:([^,\"]*)/) {

            $channel->{'aid'}           = $1;
            $channel->{'fac_type'}      = $2;
            $channel->{'state'}         = $3;
            ($channel->{'description'}) = $response =~ /ALIAS=\\"([^\\"]*)\\",/;

            push(@channels, $channel);
        }

        else {

            print STDERR "get_optical_channels: Error parsing response: $response\n";
        }
    }


    # also grab the mux/dmux stuff
    $cmd = "RTRV-OM:$tid" . ":ALL:$ctag;";

    $cmd_output = $self->cmd($cmd);
    return undef if (!defined($cmd_output));

    @output_messages = $self->parse($cmd_output);
    @responses = $self->get_responses(@output_messages);

    foreach my $response (@responses) {

        my $channel;
        #OM-1-7-1:EDFA:ALIAS=\\"\\",BAND=C,VSTIMER=08-00:OOS-AU,AINS&FAF"
        if ($response =~ /^\"([^:]+):([^:]+):.*:([^,\"]+)/) {

            $channel->{'aid'}           = $1;
            $channel->{'fac_type'}      = $2;
            $channel->{'state'}         = $3;
            ($channel->{'description'}) = $response =~ /ALIAS=\\"([^\\"]*)\\",/;

            push(@channels, $channel);
        }

        else {

            print STDERR "get_optical_channels: Error parsing response: $response\n";
        }
    }

    return @channels;
}

sub get_alarms {

    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined $cmd_output);

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);
    my @alarms;

    foreach my $response (@responses) {

        my $alarm;

        #AID:NOTIFICATION_CODE,COND_TYPE,SERV_AFFECT,DATE,TIME,LOCATION,DIRECTION:DESCRIPTION
        # "SHELF-1,EQPT:MN,PWRPROTNA,NSA,11-01,22-04-43,NEND,NA:\"Power Unit Missing\""

        if ($response =~ /^\"([^,]+),([^:]+):([^,]+),([^,]+),([^,]+),([^,]+),([^:]+),([^:]+),([^:]+):(.+)$/) {

            $alarm->{'aid'}         = $1;
            $alarm->{'aid_type'}    = $2;
            $alarm->{'not_code'}    = $3;
            $alarm->{'cond_type'}   = $4;
            $alarm->{'serv_affect'} = $5;
            $alarm->{'date'}        = $6;
            $alarm->{'time'}        = $7;
            $alarm->{'location'}    = $8;
            $alarm->{'direction'}   = $9;
            $alarm->{'description'} = $10;

            # strip the \"
            ($alarm->{'description'}) = $alarm->{'description'} =~ /\\"([^\\"]+)/;

            my @date = split(/-/, $alarm->{'date'});

            if (@date == 3) {
                ($alarm->{'year'}, $alarm->{'month'}, $alarm->{'day'}) = @date;
            }
            else {
                ($alarm->{'month'}, $alarm->{'day'}) = @date;
            }

            $alarm->{'time'} =~ s/-/:/g;

            push(@alarms, $alarm);
        }

        else {

            print STDERR "get_alarms: Error parsing response: $response\n";
        }
    }

    return @alarms;
}

sub get_inventory {

    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-EQPT:$tid" . "::$ctag;";

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined $cmd_output);

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    my @inventory;

    foreach my $response (@responses) {

        my $device;

        if ($response =~ /^\"([^:]+):([^:]*):.*:([^,:]+).*\"$/) {

            $device->{'aid'}   = $1;
            $device->{'type'}  = $2;
            $device->{'state'} = $3;

            ($device->{'serial'})        = $response =~ /SERNO=([^,:]*)/;
            ($device->{'unique_serial'}) = $response =~ /USI=([^,:]*)/;
            ($device->{'clei'})          = $response =~ /CLEI=([^,:]*)/;
            ($device->{'vendor_id'})     = $response =~ /VENDID=([^,:]*)/;
            ($device->{'unit_name'})     = $response =~ /UNITNAME=([^,:]*)/;

            if (!$device->{'type'}) {
                next;
            }
            push(@inventory, $device);
        }

        else {
            print STDERR "get_inventory: Error parsing response: $response\n";
        }
    }

    return @inventory;
}

sub get_autonomous_output {

    my ($self, $type, $start, $end) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-AO:$tid:";

    if (defined($start) && defined($end)) {
        $cmd .= "$start&&$end";
    }

    $cmd .= ":$ctag" . ":::";

    # append the MSGTYPE if needed
    if (defined($type)) {
        $cmd .= "MSGTYPE=$type,";
    }

    # lets always do descending order
    $cmd .= "ORDER=DECR;";

    $self->send($cmd);
    #  my $cmd_output = $self->cmd($cmd);
    my $cmd_output = $self->recv($cmd, $ctag);
    return undef if (!defined($cmd_output));

    #my @output_messages = $self->parse($cmd_output);
    my @messages = $self->parse_autonomous_messages($cmd_output);

    #my @responses = $self->get_responses(@output_messages);

    my @results;

    # go through each auto message
    foreach my $message (@messages) {

        next if ($message->{'verb'} !~ /^REPT $type/);

        my $responses = $message->{'responses'};

        foreach my $response (@$responses) {

            #AID:NOTIFICATION_CODE,COND_TYPE,SERV_AFFECT,DATE,TIME:DESCR
            # "SC-1-A-C1:CL,LINKDOWN,NSA,11-20,18-35-54,:\\"External Link Failure\\""
            # "CH-1-9-C1:MN,T-OPR-HT,NSA,10-02,15-24-24,NEND,RCV:\"Opt. Power Rx Too High\""
            if ($response =~ /^\"([^:]+):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*:(.+)$/) {

                my $msg;

                $msg->{'aid'}         = $1;
                $msg->{'not_code'}    = $2;
                $msg->{'cond_type'}   = $3;
                $msg->{'serv_affect'} = $4;
                $msg->{'date'}        = $5;
                $msg->{'time'}        = $6;
                $msg->{'description'} = $7;

                $msg->{'time'} =~ s/-/:/g;

                # strip the \"
                ($msg->{'description'}) = $msg->{'description'} =~ /\\"([^\\"]+)/;

                my @date = split(/-/, $msg->{'date'});

                if (@date == 3) {
                    ($msg->{'year'}, $msg->{'month'}, $msg->{'day'}) = @date;
                }
                else {
                    ($msg->{'month'}, $msg->{'day'}) = @date;
                }

                push(@results, $msg);
            }

            else {

                print STDERR "get_autonomous_output: Error parsing response: $response\n";
            }
        }
    }

    return @results;
}

sub get_optical_channel_performance {

    my ( $self, $aid, $montype, $monlev, $locn, $direction, $tmper, $index ) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    # optional args
    $tid = "" if ( !defined( $tid ) );
    $aid = "ALL" if ( !defined( $aid ) );
    $montype = "" if ( !defined( $montype ) );
    $monlev = "" if ( !defined( $monlev ) );
    $locn = "" if ( !defined( $locn ) );
    $direction = "" if ( !defined( $direction ) );
    $tmper = "" if ( !defined( $tmper ) );
    $index = "" if ( !defined( $index ) );

    my $cmd = "RTRV-PM-CH:$tid:" . "$aid:$ctag" . "::$montype,$monlev,$locn,$direction,$tmper,,,$index;";

    my $cmd_output = $self->cmd( $cmd );
    return undef if ( !defined( $cmd_output ) );

    my @output_messages = $self->parse( $cmd_output );
    my @responses = $self->get_responses( @output_messages );
    my @performance_data;

    foreach my $response ( @responses ) {

        my $data;

        # "CH-1-9-NW,CH:BYTERCV,301313040436,TRUE,NEND,NA,1-DAY,08-03,00-00-00,0"
        if ( $response =~ /^"([^,]+),([^:]+):([^,]+),([^,]+),/ ) {

            $data->{'aid'} = $1;
            $data->{'aid_type'} = $2;
            $data->{'mon_type'} = $3;
            $data->{'mon_value'} = $4;

            push( @performance_data, $data );
        }

        else {

            print STDERR "get_optical_channel_performance: Error parsing response: $response\n";
        }
    }

    return @performance_data;
}

sub get_optical_multiplex_performance {

    my ( $self, $aid, $montype, $monlev, $locn, $direction, $tmper, $index ) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    # optional args
    $tid = "" if ( !defined( $tid ) );
    $aid = "ALL" if ( !defined( $aid ) );
    $montype = "" if ( !defined( $montype ) );
    $monlev = "" if ( !defined( $monlev ) );
    $locn = "" if ( !defined( $locn ) );
    $direction = "" if ( !defined( $direction ) );
    $tmper = "" if ( !defined( $tmper ) );
    $index = "" if ( !defined( $index ) );

    my $cmd = "RTRV-PM-OM:$tid:" . "$aid:$ctag" . "::$montype,$monlev,$locn,$direction,$tmper,,,$index;";

    my $cmd_output = $self->cmd( $cmd );
    return undef if ( !defined( $cmd_output ) );

    my @output_messages = $self->parse( $cmd_output );
    my @responses = $self->get_responses( @output_messages );
    my @performance_data;

    foreach my $response ( @responses ) {

        my $data;

        # "CH-1-9-NW,CH:BYTERCV,301313040436,TRUE,NEND,NA,1-DAY,08-03,00-00-00,0"
        if ( $response =~ /^"([^,]+),([^:]+):([^,]+),([^,]+),/ ) {

            $data->{'aid'} = $1;
            $data->{'aid_type'} = $2;
            $data->{'mon_type'} = $3;
            $data->{'mon_value'} = $4;

            push( @performance_data, $data );
        }

        else {

            print STDERR "get_optical_multiplex_performance: Error parsing response: $response\n";
        }
    }

    return @performance_data;
}

sub backup_database {

    my ( $self, $ftp_user, $ftp_pass, $ftp_host, $ftp_path ) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    # first we need to copy the database from working memory to the RDISK
    my $cmd = "CPY-MEM:$tid:ACT,,RDISK,:$ctag:" . ":DBS;";

    # send the command and get the output
    my $cmd_output = $self->cmd( $cmd );

    # return if error
    return if ( !defined( $cmd_output ) );

    # now we need to find the most recent file on the RDISK
    $cmd = "RTRV-FILE:$tid:ALL,RDISK:$ctag;";

    # get output
    $cmd_output = $self->cmd( $cmd );

    # error retrieving
    return if ( !defined( $cmd_output ) );

    # parse
    my @output_messages = $self->parse( $cmd_output );
    my @responses = $self->get_responses( @output_messages );

    # store the most recent filename and its date
    my $recent_filename;
    my $recent_timestamp;

    foreach my $response ( @responses ) {

        # "F7011014_130122_0507_10_2_0_17.DBS,RDISK::SIZE=91306,CRDAT=13-01-22,CRTIM=05-07-02,GDBISSUE=11-01-4,GRADE=B,COMMENT=\"(no comment avail.)\""
        # "supportdata.tgz,RDISK::SIZE=521598,CRDAT=13-01-24,CRTIM=15-55-41"
        if ( $response =~ /^"([^,]+),RDISK::.*CRDAT=([^,]+),CRTIM=([^,"]+)/ ) {

            my $filename = $1;
            my $date = $2;
            my $time = $3;

            # only handle files that end in .DBS
            next if ( $filename !~ /\.DBS$/ );

            my ( $year, $month, $day ) = split( '-', $date );
            my ( $hour, $min, $sec ) = split( '-', $time );

            # year 2100 bug :)
            $year += 2000;

            my $timestamp;

            eval {

                $timestamp = Mktime( $year, $month, $day, $hour, $min, $sec );
            };

            if ( !$timestamp ) {

                print STDERR "backup_database: Unable to determine timestamp for $year / $month / $day / $hour / $min / $sec\n";
                next;
            }

            # set it as the most recent if it is
            if ( !defined( $recent_timestamp ) ||
                 $timestamp > $recent_timestamp ) {

                $recent_timestamp = $timestamp;
                $recent_filename = $filename;
            }
        }

        else {

            print STDERR "backup_database: Error parsing response: $response\n";
        }
    }

    # unable to find a recent filename
    return if ( !defined( $recent_filename ) );

    # now copy from RDISK to FTP server
    $cmd = "CPY-FILE:$tid:RDISK,$recent_filename,$ftp_host,$recent_filename:$ctag" . ":::FTPATH=\"$ftp_path\",FTPID=\"$ftp_pass\",FTUID=\"$ftp_user\",PROTOCOL=FTP;";
    $cmd_output = $self->cmd( $cmd );

    my $output_message;

    if ( $cmd_output ) {

        ( $output_message ) = $self->parse( $cmd_output );
    }

    # error retrieving
    if ( !defined( $cmd_output ) || $output_message->{'completion_code'} ne "COMPLD" ) {

        # clean up copied file off of RDISK
        $self->_delete_file( $recent_filename );
        return;
    }

    # delete the file from the RDISK now that we're done with it
    return $self->_delete_file( $recent_filename );
}

sub backup_alarm_profile {

    my ( $self, $ftp_user, $ftp_pass, $ftp_host, $ftp_path ) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    # first we need to copy the database from working memory to the RDISK
    my $cmd = "CPY-MEM:$tid:ACT,,RDISK,:$ctag:" . ":ALP;";

    # send the command and get the output
    my $cmd_output = $self->cmd( $cmd );

    # return if error
    return if ( !defined( $cmd_output ) );

    # now we need to find the most recent file on the RDISK
    $cmd = "RTRV-FILE:$tid:ALL,RDISK:$ctag;";

    # get output
    $cmd_output = $self->cmd( $cmd );

    # error retrieving
    return if ( !defined( $cmd_output ) );

    # parse
    my @output_messages = $self->parse( $cmd_output );
    my @responses = $self->get_responses( @output_messages );

    # store the most recent filename and its date
    my $recent_filename;
    my $recent_timestamp;

    foreach my $response ( @responses ) {

        # "F7011014_130122_0507_10_2_0_17.DBS,RDISK::SIZE=91306,CRDAT=13-01-22,CRTIM=05-07-02,GDBISSUE=11-01-4,GRADE=B,COMMENT=\"(no comment avail.)\""
        # "supportdata.tgz,RDISK::SIZE=521598,CRDAT=13-01-24,CRTIM=15-55-41"
        # "F7011014_130205_2018_10_2_0_14.ALP,RDISK::SIZE=614,CRDAT=13-02-05,CRTIM=20-18-26,GDBISSUE=11-01-4,COMMENT=\"Empty comment.\""
        if ( $response =~ /^"([^,]+),RDISK::.*CRDAT=([^,]+),CRTIM=([^,"]+)/ ) {

            my $filename = $1;
            my $date = $2;
            my $time = $3;

            # only handle files that end in .ALP
            next if ( $filename !~ /\.ALP$/ );

            my ( $year, $month, $day ) = split( '-', $date );
            my ( $hour, $min, $sec ) = split( '-', $time );

            # year 2100 bug :)
            $year += 2000;

            my $timestamp;

            eval {

                $timestamp = Mktime( $year, $month, $day, $hour, $min, $sec );
            };

            if ( !$timestamp ) {

                print STDERR "backup_alarm_profile: Unable to determine timestamp for $year / $month / $day / $hour / $min / $sec\n";
                next;
            }

            # set it as the most recent if it is
            if ( !defined( $recent_timestamp ) ||
                 $timestamp > $recent_timestamp ) {

                $recent_timestamp = $timestamp;
                $recent_filename = $filename;
            }
        }

        else {

            print STDERR "backup_alarm_profile: Error parsing response: $response\n";
        }
    }

    # unable to find a recent filename
    return if ( !defined( $recent_filename ) );

    # now copy from RDISK to FTP server
    $cmd = "CPY-FILE:$tid:RDISK,$recent_filename,$ftp_host,$recent_filename:$ctag" . ":::FTPATH=\"$ftp_path\",FTPID=\"$ftp_pass\",FTUID=\"$ftp_user\",PROTOCOL=FTP;";
    $cmd_output = $self->cmd( $cmd );

    my $output_message;

    if ( $cmd_output ) {

        ( $output_message ) = $self->parse( $cmd_output );
    }

    # error retrieving
    if ( !defined( $cmd_output ) || $output_message->{'completion_code'} ne "COMPLD" ) {

        # clean up copied file off of RDISK
        $self->_delete_file( $recent_filename );
        return;
    }

    # delete the file from the RDISK now that we're done with it
    return $self->_delete_file( $recent_filename );
}

sub _delete_file {

    my ( $self, $filename ) = @_;

    my $tid = $self->{'tid'};
    my $ctag = $self->{'ctag'};

    my $cmd = "DLT-FILE:$tid:$filename,RDISK:$ctag;";
    my $cmd_output = $self->cmd( $cmd );

    return if ( !defined( $cmd_output ) );

    my ( $output_message ) = $self->parse( $cmd_output );

    return $output_message->{'completion_code'} eq "COMPLD";
}

1;
