#--------------------------------------------------------------------
#----- GRNOC TL1 Ciena ActivFlex 6500 Device Module
#-----
#----- Copyright(C) 2011 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module handles the functionality for the Ciena ActivFlex
#----- 6500 device.  It inherts from the standard GRNOC::TL1::Device
#----- module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Ciena::ActivFlex6500;

use strict;
use warnings;

use base "GRNOC::TL1::Device";

use Date::Calc qw(Monday_of_Week Weeks_in_Year);
use Data::Dumper;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;

  my (%args) = @_;

  # set default port used for 6500 if one was not given
  $args{'port'} = 23 if ($args{'port'} eq '');

  # set default prompt used for 6500 if one was not given
  $args{'prompt'} = ';' if ($args{'prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);

  return $self;
}

sub login {

  my ($self, @args) = @_;

  # store the current prompt
  my $old_prompt = $self->{'prompt'};

  # set the prompt to be a < char while we login
  $self->{'prompt'} = '<';

  # call the GRNOC::TL1::Device login method as normal now
  my $result = $self->SUPER::login(@args);

  # restore the old prompt
  $self->{'prompt'} = $old_prompt;

  # just return now if it failed to login
  return $result if (!$result);

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  # we should also disable autonomous messages from mucking up our output
  my $cmd = "INH-MSG-ALL:$tid" . "::$ctag;";
  my $cmd_output = $self->cmd($cmd);

  # check for error
  return undef if (!defined($cmd_output));

  # return whatever the main login() method returned
  return $result;
}

sub get_performance_data {

  my ($self, $aid) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-PM-ALL:$tid" . ":$aid:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @performance_data;

  foreach my $response (@responses) {

    my $data;

    if ($response =~ /^"([^,]+),([^:]*):([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),(.*)"$/) {

      $data->{'aid'} = $1;
      $data->{'aid_type'} = $2;
      $data->{'mon_type'} = $3;
      $data->{'mod_value'} = $4;
      $data->{'validity'} = $5;
      $data->{'location'} = $6;
      $data->{'direction'} = $7;
      $data->{'time_period'} = $8;
      $data->{'date'} = $9;
      $data->{'time'} = $10;
      $data->{'index'} = $11;
      $data->{'physical_aid'} = $12;
      $data->{'channel_id'} = $13;
      $data->{'channel_status'} = $14;

      push(@performance_data, $data);
    }

    else {

      print STDERR "get_performance_data: Error parsing response: $response\n";
    }
  }

  return @performance_data;
}

sub get_optical_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-AMP:$tid" . ":ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @interfaces;

  foreach my $response (@responses) {

    my $interface;

    if ($response =~ /^"([^:]+)::.*:([^"]+)"$/) { #"

      $interface->{'aid'} = $1;
      $interface->{'state'} = $2;

      ($interface->{'mode'}) = $response =~ /AMPMODE=([^,:]*)/;
      ($interface->{'rx_loss'}) = $response =~ /INPUTLOSS=([^,:]*)/;
      ($interface->{'tx_loss'}) = $response =~ /OUTPUTLOSS=([^,:]*)/;
      ($interface->{'gain'}) = $response =~ /[,:]GAIN=([^,:]*)/;

      push(@interfaces, $interface);
    }

    else {

      print STDERR "get_optical_interfaces: Error parsing response: $response\n";
    }
  }

  return @interfaces;
}

sub get_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ETHN:$tid" . ":ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @interfaces;

  foreach my $response (@responses) {

    my $interface;

    # "ETH100G-1-31-1::LASEROFFFARENDFAIL=DISABLED,MAPPING=PROPRIETARY,OCHTXACTHIGHPWR=\"0.8\",OCHTXACTLOWPWR=\"0.2\",OCHTXMINPWR=\"-6.3\",OCHTXMAXPWR=\"6.4\",OCHRXACTHIGHPWR=\"OOR-LO\",OCHRXACTLOWPWR=\"OOR-LO\",OCHRXMINPWR=\"-12.5\",OCHRXMAXPWR=\"4.5\",CLFI=\"\",AINSTIMELEFT=00-05,,:OOS-AU,FLT&DISCD&AINS"
    if ($response =~ /^"([^:]+)::.*:([^"]+)"$/) { #"

      $interface->{'aid'} = $1;
      $interface->{'state'} = $2;

      ($interface->{'description'}) = $response =~ /CLFI=\\"([^\\"]*)\\",/;

      push(@interfaces, $interface);
    }

    else {

      print STDERR "get_ethernet_interfaces: Error parsing response: $response\n";
    }
  }

  # RTRV-ETHN returns only the 100G interfaces, now we need to grab the 10G ones as well
  $cmd = "RTRV-ETH10G:$tid" . ":ALL:$ctag;";

  $cmd_output = $self->cmd($cmd);
  return undef if (! defined ($cmd_output));

  @output_messages = $self->parse($cmd_output);
  @responses = $self->get_responses(@output_messages);  

  foreach my $response (@responses){
    my $interface;

    # "ETH10G-2-6-1::LASEROFFFARENDFAIL=DISABLED,OCHTXACTPWR=\"-3.1\",OCHTXMINPWR=\"-8.0\",OCHTXMAXPWR=\"0.9\",OCHRXACTPWR=\"-3.5\",OCHRXMINPWR=\"-13.0\",OCHRXMAXPWR=\"0.4\",MTU=9600,FLOWCTRL=NONE,MAPPING=GFPMACTR,CLFI=\"\",,:IS"
    if ($response =~ /^"([^:]+)::.*:([^"]+)"$/) { #"

      $interface->{'aid'} = $1;
      $interface->{'state'} = $2; 

      ($interface->{'description'}) = $response =~ /CLFI=\\"([^\\"]*)\\",/;

      push(@interfaces, $interface);
    }

    else {

      print STDERR "get_ethernet_interfaces: Error parsing response: $response\n";
    }
  }

  return @interfaces;
}

sub get_oc192_interfaces {
    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid  = $self->{'tid'};

    my $cmd  = "RTRV-OC192:$tid:ALL:$ctag;";

    my $cmd_output = $self->cmd($cmd);
    return undef if (! defined $cmd_output);

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);
    
    my @interfaces;

    foreach my $response (@responses){
	my $interface;

	# OC192-1-5-1::,,,SDTH=6,,,,,,EBERTH=3,PORTMODE=SONET,LASEROFFFARENDFAIL=DISABLED,OCHTXACTPWR=\"-2.8\",OCHTXMINPWR=\"-8.0\",OCHTXMAXPWR=\"0.9\",OCHRXACTPWR=\"-7.4\",OCHRXMINPWR=\"-13.0\",OCHRXMAXPWR=\"0.4\",CLFI=\"\":IS,"
	if ($response =~ /^"([^:]+):.*:([^,]*),.*"$/) {
	    
	    $interface->{'aid'}     = $1;
	    $interface->{'state'}   = $2;

	    ($interface->{'mode'})        = $response =~ /PORTMODE=([^,]+)/;
	    ($interface->{'description'}) = $response =~ /CLFI=\\"([^\\"]*)\\",/;

	    push (@interfaces, $interface);

	}
	else {
	    print STDERR "get_oc192_interfaces: Error parsing response: $response\n";
	}

    }

    return @interfaces;
}

sub get_otm_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my @cmds = ("RTRV-OTM:$tid" . ":ALL:$ctag;",
              "RTRV-OTM2:$tid" . ":ALL:$ctag;",
              "RTRV-OTM3:$tid" . ":ALL:$ctag;");

  my @interfaces;

  foreach my $cmd (@cmds) {

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined($cmd_output));

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    foreach my $response (@responses) {

      my $interface;

# "OTM3-2-22-1::CLFI=\"\",PORTMODE=SONET,OTURATE=44G5,TFMODE=OFF,OTUTIMEN=OPERATOR,OSID=\"\",OTUTXFECFRMT=PFEC,OTURXFECFRMT=PFEC,OCHRXACTDISP=\"5300\",OCHTXPWR=\"0.0\",OCHTXACTPWR=\"0.0\",OCHTXMINPWR=\"-11.0\",OCHTXMAXPWR=\"0.0\",OCHRXACTPWR=\"-10.2\",OCHRXMINPWR=\"-22.5\",OCHRXMAXPWR=\"7.5\",OCHTXWVLNGTHPROV=\"1547.72\",OCHTXWVLNGTHMIN=\"1527.60\",OCHTXWVLNGTHMAX=\"1565.50\",OCHTXWVLNGTHSPACING=50,TUNINGMODE=NORMAL,OCHRXECHOTRACE=\"ATLDGAUYATL2CD6I2355 02-22-001\",OCHTXTRACE=\"ATLDGAUYATL2CD6I2355 02-22-001\",OCHTXASSOCFARENDRX=\"MCLNVA23WDC1CD6I2308 02-22-001\",OCHDIFFERENTIALENCODING=NONE,ODUTFMODE=OFF,PREFECSFTHLEV=\"0.00\",PREFECSFTHBER=\"3.80E-03\",PREFECSDTHLEV=\"0.50\",PREFECSDTHBER=\"1.36E-03\",ODUTTITERM=NO,ODUMONITOR=NO,OCHMAXPMD=\"10\",OCHRXACTPMD=\"2\",OCHREACHSPEC=\"1800\",OCHESTLENGTH=\"1298\":IS,"
      if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

        $interface->{'aid'} = $1;
        $interface->{'state'} = $2;

        ($interface->{'mode'}) = $response =~ /PORTMODE=([^,]*),/;
        ($interface->{'description'}) = $response =~ /CLFI=\\"([^\\"]*)\\",/;

        push(@interfaces, $interface);
      }

      else {

        print STDERR "get_otm_interfaces: Error parsing response: $response\n";
      }
    }
  }

  return @interfaces;
}

sub get_optical_channels {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-CHC:$tid" . ":ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @channels;

  foreach my $response (@responses) {

    my $channel;

    # "CHC-1-23-22-156305::ISOPQ=NO,TARGLOSS=0.00,SWSEL=19,INPOW=-9.00,INPOWSRC=\"I2LAB1-1-53-77\",OUTPOW=-14.48,LOSS=0.41,DRIVE=0.00:IS,"
    if ($response =~ /^"([^:]+)::ISOPQ=([^,]*),TARGLOSS=([^,]*),SWSEL=(\d*).*"$/) {

      $channel->{'aid'} = $1;
      $channel->{'is_opaque'} = $2;
      $channel->{'target_loss'} = $3;
      $channel->{'switch_selector'} = $4;

      ($channel->{'rx_power'}) = $response =~ /INPOW=([^,:]*)/;
      ($channel->{'rx_power_source'}) = $response =~ /INPOWSRC=([^,:]*)/;
      ($channel->{'tx_power'}) = $response =~ /OUTPOW=([^,:]*)/;
      ($channel->{'loss'}) = $response =~ /[,:]LOSS=([^,:]*)/;
      ($channel->{'drive'}) = $response =~ /DRIVE=([^,:]*)/;
      ($channel->{'state'}) = $response =~ /:([^"]+)"$/;

      # strip \ and "s
      $channel->{'rx_power_source'} =~ s/\\//g;
      $channel->{'rx_power_source'} =~ s/"//g;

      push(@channels, $channel);
    }

    else {

      print STDERR "get_optical_channels: Error parsing response: $response\n";
    }
  }

  return @channels;
}

sub get_optical_service_channels {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-OSC:$tid" . ":ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);

  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @oscs;

  foreach my $response (@responses) {

    my $osc;

    #"OSC-1-1-1::RXPATHLOSS=0.70,OSCTXPOW=-5.36,OSCTXWAVELENGTH=1511.00,OSCESTBER=0.0E+00,SDTH=6,OSCRECEIVEPOW=-19.02:IS,"
    #"OSC-1-18-1::RXPATHLOSS=0.70,OSCTXPOW=-5.68,OSCTXWAVELENGTH=1511.00,OSCSPANLOSS=21.98,OSCESTBER=0.0E+00,SDTH=6,OSCRECEIVEPOW=-29.66:IS,"
    if ($response =~ /^"([^:]+)::.*:([^"]+)"$/) {

      $osc->{'aid'} = $1;
      $osc->{'state'} = $2;

      ($osc->{'tx_power'}) = $response =~ /OSCTXPOW=([^,]*),/;
      ($osc->{'rx_power'}) = $response =~ /OSCRECEIVEPOW=([^,:]*)/;
      ($osc->{'ber'}) = $response =~ /OSCESTBER=([^,]*),/;
      ($osc->{'rx_loss'}) = $response =~ /RXPATHLOSS=([^,]*),/;
      ($osc->{'span_loss'}) = $response =~ /OSCSPANLOSS=([^,]*),/;
      ($osc->{'wavelength'}) = $response =~ /OSCTXWAVELENGTH=([^,]*),/;
    }

    else {

      print STDERR "get_optical_service_channels: Error parsing response: $response\n";
    }

    push(@oscs, $osc);
  }

  return @oscs;
}

sub get_alarms {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /^"([^,]*),([^:]*):([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^:]*):\\\"([^\\]*)\\\",[^:]*:([^-]*)-([^-]*)-([^,]*),:YEAR=([^,]*),MODE=([^\"]*)\"/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'month_day'} = $6;
      $alarm->{'time'} = $7;
      $alarm->{'location'} = $8;
      $alarm->{'direction'} = $9;
      $alarm->{'description'} = $10;
      $alarm->{'id'} = $11;
      $alarm->{'prob_cause'} = $12;
      $alarm->{'doc_index'} = $13;
      $alarm->{'year'} = $14;
      $alarm->{'mode'} = $15;

      my @two = split(/-/, $alarm->{'month_day'});
      $alarm->{'month'} = $two[0];
      $alarm->{'day'} = $two[1];

      $alarm->{'date'} = "$alarm->{'month'}-$alarm->{'day'}-$alarm->{'year'}";

      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing response: $response\n";
    }
  }

  return @alarms;
}

sub get_inventory {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-INVENTORY:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  # we also need to do a RTRV-BACKPLANE
  $cmd = "RTRV-BACKPLANE:$tid" . ":ALL:$ctag;";
  $cmd_output = $self->cmd($cmd);

  # error retreiving
  return undef if (!defined($cmd_output));

  my @bp_output_messages = $self->parse($cmd_output);
  my @bp_responses = $self->get_responses(@bp_output_messages);

  # combine the two responses into one
  @responses = (@bp_responses, @responses);

  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^:]+)::.*"$/) {

      $device->{'aid'} = $1;

      # skip over it if it doesn't have a CTYPE (empty & filler slots)
      next if ($response !~ /CTYPE=/);

      ($device->{'type'}) = $response =~ /CTYPE=\\"([^\\"]+)\\"/;
      ($device->{'snmp_index'}) = $response =~ /SNMPINDEX=([^,|^"]+)/;
      ($device->{'pec'}) = $response =~ /PEC=([^,|^"]+)/;
      ($device->{'version'}) = $response =~ /REL=([^,|^"]+)/;
      ($device->{'clei'}) = $response =~ /CLEI=([^,|^"]+)/;
      ($device->{'serial'}) = $response =~ /SER=([^,|^"]+)/;
      ($device->{'date'}) = $response =~ /MDAT=([^,|^"]+)/;

      $device->{'version'} =~ s/^\s//;
      $device->{'version'} =~ s/\s$//;

      # fix bogus serial numbers
      if ( $device->{'pec'} eq 'NTT862DA' ||
           $device->{'pec'} eq 'NTT862BB' ||
           $device->{'pec'} eq 'NTT862BA' ) {

        # they might have an extra LB prepended to the actual serial number...
        if ( $device->{'serial'} =~ /^LB(.+)/ ) {

          $device->{'serial'} = $1;
        }
      }

      # reformat date to be month-day-year if needed
      if ($device->{'date'}) {

        my ($year, $week_num) = split(/-/, $device->{'date'});

        # check for bogus year
        if ( $year < 1990 ) {

          $device->{'date'} = undef;
        }

        # year should be okay
        else {

          # did they give us a bogus week #? (yes this has happened)
          if ($week_num < 1 || $week_num > Weeks_in_Year($year)) {

            # make something up i guess
            $week_num = Weeks_in_Year($year);
          }

          my $month;
          my $day;

          ($year, $month, $day) = Monday_of_Week($week_num, $year);

          $device->{'date'} = "$month-$day-$year";
        }
      }

      # there seem to be lots of cases where there is leading and trailing whitespace so lets go ahead and remove all such cases
      my @keys = keys(%$device);

      foreach my $key (@keys) {

        my $value = $device->{$key};

        if ($value) {

          $value =~ s/^\s*//g;
          $value =~ s/\s*$//g;

          $device->{$key} = $value;
        }
      }

      push(@inventory, $device);
    }

    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  return @inventory;
}

sub get_cross_connects {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-CRS-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @cross_connects;

  foreach my $response (@responses) {

    my $cross_connect;

    if ($response =~ /^"([^,]+),([^:]+):([^:]*):.*:([^:]+):"$/) {

      $cross_connect->{'from_aid'} = $1;
      $cross_connect->{'to_aid'} = $2;
      $cross_connect->{'type'} = $3;
      $cross_connect->{'rate'} = $4;

      ($cross_connect->{'description'}) = $response =~ /CKTID=([^,]*),/;

      push(@cross_connects, $cross_connect);
    }

    else {

      print STDERR "get_cross_connects: Error parsing response: $response\n";
    }
  }

  return @cross_connects;
}

sub get_adjacencies {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ADJ:$tid" . ":ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @adjs;

  foreach my $response (@responses) {

    my $adj = {};

    # "ADJ-1-1-1::ADJTYPE=LIM,PROVFEADDR=\"MNJTNJABPRN1CA6I2303-1-2-3\",PADDRFORM=TID-SH-SL-PRT,DISCFEADDR=\"\",DADDRFORM=NULL,ADJSTAT=DERIVED,CLFI=\"\":IS,"
    if ($response =~ /^"([^:]+)::ADJTYPE=([^,]+),.*CLFI=\\"([^\\"]*)\\":([^"]+)"$/) {

      $adj->{'aid'} = $1;
      $adj->{'type'} = $2;
      $adj->{'description'} = $3;
      $adj->{'state'} = $4;

      push(@adjs, $adj);
    }

    else {

      print STDERR "get_adjacencies: Error parsing response: $response\n";
    }
  }

  return @adjs;
}

sub get_network_element_type {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-NETYPE:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  foreach my $response (@responses) {

    my $ne_type = {};

    if ($response =~ /^([^,]+),([^,]+),([^,]+),(.+)"$/) {

      my ($manufacturer, $model, $type, $software) = ($1, $2, $3, $4);

      # strip \"
      $model =~ s/\\"//g;
      $software =~ s/\\"//g;

      $ne_type->{'manufacturer'} = $manufacturer;
      $ne_type->{'model'} = $model;
      $ne_type->{'type'} = $type;
      $ne_type->{'software'} = $software;

      return $ne_type;
    }

    else {

      print STDERR "get_network_element_type: Error parsing response: $response\n";
    }
  }

  return undef;
}

sub get_autonomous_output {

  my ($self, $type, $start, $end) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-AO:$tid:";

  my $count;

  # determine the total COUNT if start & end given
  if (defined($start) && defined($end)) {

    $count = abs($end - $start);
  }

  $cmd .= ":$ctag" . ":::,";

  # append the COUNT if needed
  if (defined($count)) {

    $cmd .= "COUNT=$count,";
  }

  else {

    $cmd .= ",";
  }

  # append the MSGTYPE if needed
  if (defined($type)) {

    $cmd .= "MSGTYPE=$type,";
  }

  else {

    $cmd .= ",";
  }

  # lets always do descending order
  $cmd .= "ORDER=DESCENDING;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined($cmd_output));

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @results;

  # go through each auto message
  foreach my $response (@responses) {

    # "ATAGSEQ=0000004845,MSGTYPE=ALM,SUBTYPE=COM:SHELF-2:MJ,MBR_UNKNOWN,NSA,01-18,11-31-08,NEND,NA:\"Member Shelf Unknown\",NONE:0100000021-6286-0719,:YEAR=2004,MODE=NONE"
    if ($response =~ /^"ATAGSEQ=(\d+),MSGTYPE=([^,]+),SUBTYPE=([^:]*):([^:]*):(.*):YEAR=([^,]*),.*$/) {

      my $result;

      $result->{'atagseq'} = $1;
      $result->{'msg_type'} = $2;
      $result->{'sub_type'} = $3;
      $result->{'aid'} = $4;
      $result->{'year'} = $6;

      my $rest = $5;

      # did it give us back msg types that we didn't ask for?
      next if (defined($type) && $result->{'msg_type'} ne $type);

      # handle ALM messages separately from EVT
      if ($result->{'msg_type'} eq "ALM") {

        $rest =~ /^([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^:]+):([^,]+),([^:]+):([^,]+),/;

        $result->{'not_code'} = $1;
        $result->{'cond_type'} = $2;
        $result->{'serv_affect'} = $3;
        $result->{'date'} = $4;
        $result->{'time'} = $5;
        $result->{'location'} = $6;
        $result->{'direction'} = $7;
        $result->{'description'} = $8;
        $result->{'mode'} = $9;
        $result->{'identifier'} = $10;

        $result->{'time'} =~ s/-/:/g;
      }

      # SHELF-1:BKUPC,TC,01-22,11-37-02,NEND,NA,,,:\"Database Save Completed\",NONE:0100000000-0000-0196,:YEAR=2004,MODE=NONE"
      # CHMON-1-22-5-153033:T-OPT-OCH,TC,01-17,11-41-45,NEND,TRMT,-29.80,6,1-UNT:\"T-OPT-OCH\":0100000000-0000-0000,:YEAR=2004,MODE=NONE,WAVELENGTH=1530.33"
      else {

        $rest =~ /^([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),(.*)/;

        $result->{'cond_type'} = $1;
        $result->{'not_code'} = $2;
        $result->{'date'} = $3;
        $result->{'time'} = $4;
        $result->{'location'} = $5;
        $result->{'direction'} = $6;

        my $rest2 = $7;

        my @fields = split(/[,:]/, $rest2);

        # sigh.. output format is different depending upon the type of event..
        if (@fields == 5) {

          $result->{'description'} = $fields[3];
          $result->{'identifier'} = $fields[4];
        }

        else {

          $result->{'description'} = $fields[3];
          $result->{'mode'} = $fields[4];
          $result->{'identifier'} = $fields[5];
        }
      }

      # strip \ and "s
      $result->{'description'} =~ s/\\//g;
      $result->{'description'} =~ s/"//g;

      $result->{'time'} =~ s/-/:/g;
      $result->{'date'} .= "-$result->{'year'}";

      push(@results, $result);
    }

    else {

      print STDERR "get_autonomous_output: Error parsing response: $response\n";
    }
  }

  return @results;
}

1;
