#--------------------------------------------------------------------
#----- GRNOC TL1 Ciena CoreDirector Device Module
#-----
#----- Copyright(C) 2008 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module handles the functionality for the Ciena Core Director TL1 device.
#----- It inherts from the standard GRNOC::TL1::Device module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Ciena::CoreDirector;
use base "GRNOC::TL1::Device";

use Data::Dumper;
use strict;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;

  my (%args) = @_;

  # set default port used for Core Director if one was not given
  $args{'port'} = 10201 if ($args{'port'} eq '');

  # set default prompt used for Core Director if one was not given
  $args{'prompt'} = ';' if ($args{'prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);

  return $self;
}

# re-use TL1::Device login method, but also inhibit autonomous messages
sub login {

  my ($self, @args) = @_;

  # call the super login method
  my $result = $self->SUPER::login(@args);

  # just return if if failed
  return $result if (!$result);

  # send the command to disable autonomous messages now
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "INH-MSG-ALL:$tid" . "::$ctag;";
  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  return $result;
}

sub get_alarms {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /.*\"([^,]*),([^:]*):([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^:]*):\\\"([^\\\"]*)\\\",([^\"]*)\".*/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'severity'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'date'} = $6;
      $alarm->{'time'} = $7;
      $alarm->{'location'} = $8;
      $alarm->{'direction'} = $9;
      $alarm->{'description'} = $10;
      $alarm->{'aid_det'} = $11;

      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }

  return @alarms;
}

sub get_inventory {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my @inventory;

  # first get information about the chassis (separate command needed)
  my $cmd = "RTRV-EQPT:$tid:1:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @chassis_response = $self->get_responses(@output_messages);

  # get all the other equipment
  $cmd = "RTRV-EQPT:$tid:ALL:$ctag;";

  $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  @output_messages = $self->parse($cmd_output);
  my @other_responses = $self->get_responses(@output_messages);

  # combine the results from the chassis and the others
  my @responses = (@chassis_response, @other_responses);

  # keep track if we found 1-A (grab it later if we don't)
  my $found_1A = 0;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^,]+),.*"$/) {

      $device->{'aid'} = $1;

      $found_1A = 1 if ($device->{'aid'} eq "1-A");

      ($device->{'type'}) = $response =~ /TYPE=([^,"]*)/;
      ($device->{'icpk_type'}) = $response =~ /ICPKTYPE=([^,"]*)/;
      ($device->{'name'}) = $response =~ /EQPTNAME=([^,"]*)/;
      ($device->{'label'}) = $response =~ /LABEL=([^,"]*)/;
      ($device->{'serial'}) = $response =~ /SRLNUM=([^,"]*)/;
      ($device->{'state'}) = $response =~ /PST=([^,"]*)/;
      ($device->{'secondary_state'}) = $response =~ /SST=\[([^\]]*)/;
      ($device->{'laser'}) = $response =~ /LASERST=([^,"]*)/;
      ($device->{'clei'}) = $response =~ /CLEI=([^,"]*)/;
      ($device->{'version'}) = $response =~ /CPKHW=([^,"]*)/;
      ($device->{'software_version'}) = $response =~ /CPKSW=([^,"]*)/;
      ($device->{'firmware'}) = $response =~ /CPKFW=([^,"]*)/;
      ($device->{'date'}) = $response =~ /MFGDAT=([^,"]*)/;
      ($device->{'ethernet_ports'}) = $response =~ /MAXETHERNETPORT=([^,"]*)/;
      ($device->{'max_rate'}) = $response =~ /MAXRATESUPPORTED=([^,"]*)/;
      ($device->{'port_config'}) = $response =~ /PORTCONFIGMODE=([^,"]*)/;

      my ($subc) = $response =~ /SUBC=([^,]*),/;
      my @sub_components = split(/&/, $subc);
      $device->{'sub_components'} = \@sub_components;

      push(@inventory, $device);
    }

    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  # grab 1-A info if not included in output
  $cmd = "RTRV-EQPT:$tid:1-A:$ctag;";

  $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  @output_messages = $self->parse($cmd_output);
  my @shelf_response = $self->get_responses(@output_messages);

  foreach my $response (@shelf_response) {

    my $device;

    if ($response =~ /^"([^,]+),.*"$/) {

      $device->{'aid'} = $1;

      ($device->{'type'}) = $response =~ /TYPE=([^,"]*)/;

      my ($subc) = $response =~ /SUBC=([^,]*),/;
      my @sub_components = split(/&/, $subc);
      $device->{'sub_components'} = \@sub_components;

      push(@inventory, $device);
    }
  }

  # do individual RTRV-EQPT for each ethernet port in an LM-10-GE-PKT
  foreach my $part (@inventory) {

    next if ($part->{'type'} ne "LM-10-GE-PKT");

    for (1 .. 10) {

      my $aid = $part->{'aid'} . "-1-" . $_;
      my $cmd = "RTRV-EQPT:$tid:$aid:$ctag;";

      $cmd_output = $self->cmd($cmd);
      return undef if (!defined $cmd_output);

      my @output_messages = $self->parse($cmd_output);
      my @responses = $self->get_responses(@output_messages);

      foreach my $response (@responses) {

        my $device;

        if ($response =~ /^"([^,]+),.*$/) {

          $device->{'aid'} = $1;

          ($device->{'type'}) = $response =~ /TYPE=([^,"]*)/;
          ($device->{'icpk_type'}) = $response =~ /ICPKTYPE=([^,"]*)/;
          ($device->{'name'}) = $response =~ /EQPTNAME=([^,"]*)/;
          ($device->{'label'}) = $response =~ /LABEL=([^,"]*)/;
          ($device->{'serial'}) = $response =~ /SRLNUM=([^,"]*)/;
          ($device->{'state'}) = $response =~ /PST=([^,"]*)/;
          ($device->{'secondary_state'}) = $response =~ /SST=\[([^\]]*)/;
          ($device->{'laser'}) = $response =~ /LASERST=([^,"]*)/;
          ($device->{'clei'}) = $response =~ /CLEI=([^,"]*)/;
          ($device->{'version'}) = $response =~ /CPKHW=([^,"]*)/;
          ($device->{'software_version'}) = $response =~ /CPKSW=([^,"]*)/;
          ($device->{'firmware'}) = $response =~ /CPKFW=([^,"]*)/;
          ($device->{'date'}) = $response =~ /MFGDAT=([^,"]*)/;
          ($device->{'ethernet_ports'}) = $response =~ /MAXETHERNETPORT=([^,"]*)/;
          ($device->{'max_rate'}) = $response =~ /MAXRATESUPPORTED=([^,"]*)/;
          ($device->{'port_config'}) = $response =~ /PORTCONFIGMODE=([^,"]*)/;

          push(@inventory, $device);
        }
      }
    }
  }

  return @inventory;
}

sub get_cross_connects {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-CRS:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @cross_connects;

  foreach my $response (@responses) {

    my $cross_connect;

    if ($response =~ /^"FROMENDPOINT=([^,]*),TOENDPOINT=([^:]*):.*::PST=([^,]*),.*"$/) {

      $cross_connect->{'from_aid'} = $1;
      $cross_connect->{'to_aid'} = $2;
      $cross_connect->{'state'} = $3;

      ($cross_connect->{'name'}) = $response =~ /NAME=([^,]*),/;
      ($cross_connect->{'description'}) = $response =~ /CKTID=([^,]*),/;
      ($cross_connect->{'alias'}) = $response =~ /ALIAS=([^,]*),/;
      ($cross_connect->{'from_type'}) = $response =~ /FROMTYPE=([^,]*),/;
      ($cross_connect->{'to_type'}) = $response =~ /TOTYPE=([^,]*),/;
      ($cross_connect->{'rate'}) = $response =~ /SIZE=([^,]*),/;

      push(@cross_connects, $cross_connect);
    }

    else {

      print STDERR "get_cross_connects: Error parsing response: $response\n";
    }
  }

  return @cross_connects;
}

sub get_optical_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-OCN:$tid:ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    my $interface;

    if ($response =~ /^"([^,]+),.*"$/) {

      $interface->{'aid'} = $1;
      ($interface->{'rate'}) = $response =~ /RATE=([^,]*)/;
      ($interface->{'description'}) = $response =~ /ALIAS=([^,]*)/;
      ($interface->{'signal'}) = $response =~ /SIGNALST=([^,]*)/;
      ($interface->{'state'}) = $response =~ /PST=([^,]*)/;

      push(@interfaces, $interface);
    }

    else {

      print STDERR "get_optical_interfaces: Error parsing response: $response\n";
    }
  }

  return @interfaces;
}

sub get_ethernet_interface {

  my ($self, $aid) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-GIGE:$tid:" . "$aid:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my $response = $responses[0];

  my $interface;

  if ($response =~ /^"([^,]+),.*"$/) {

    $interface->{'aid'} = $1;
    ($interface->{'description'}) = $response =~ /ALIAS=([^,]*)/;
    ($interface->{'state'}) = $response =~ /PST=([^,]*)/;
    ($interface->{'type'}) = $response =~ /ETHERPHY=([^,]*)/;

  }

  else {

    print STDERR "get_ethernet_interface: Error parsing response: $response\n";
  }

  return $interface;
}

sub get_ethernet_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-GIGE:$tid:" . "ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    if ($response =~ /^"([^,]+),.*"$/) {

      my $interface;

      $interface->{'aid'} = $1;
      ($interface->{'description'}) = $response =~ /ALIAS=([^,]*)/;
      ($interface->{'state'}) = $response =~ /PST=([^,]*)/;
      ($interface->{'type'}) = $response =~ /ETHERPHY=([^,]*)/;

      push(@interfaces, $interface);
    }

    else {

      print STDERR "get_ethernet_interface: Error parsing response: $response\n";
    }
  }

  return @interfaces;
}

1;
