#--------------------------------------------------------------------
#----- GRNOC TL1 Ciena CoreStream Device Module
#-----
#----- Copyright(C) 2010 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: svn+ssh://svn.grnoc.iu.edu/grnoc/perl-lib/GRNOC-TL1/tags/1.3.10/lib/GRNOC/TL1/Device/Ciena/CoreStream.pm $
#----- $Id: CoreStream.pm 11586 2011-03-22 20:20:18Z daldoyle $
#-----
#----- This module handles the functionality for the Ciena CoreStream
#----- TL1 device.  It inherts from the standard GRNOC::TL1::Device
#----- module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Ciena::CoreStream;
use base "GRNOC::TL1::Device";

use Data::Dumper;
use strict;

sub new {

    my $caller = shift;
    my $class = ref($caller) || $caller;
    
    my (%args) = @_;

    # set default port used for CoreStream if one was not given
    $args{'port'} = 3083 if ($args{'port'} eq '');

    # set default prompt used for CoreStream if one was not given
    $args{'prompt'} = ';' if ($args{'prompt'} eq '');

    # don't wait for a login prompt..
    $args{'wait_for_login_prompt'} = 0 if ($args{'wait_for_login_prompt'} eq '');

    # call the GRNOC::TL1::Device structure
    my $self = $class->SUPER::new(%args);

    bless($self, $class);

    return $self;
}

# we need to override the default connect() method a bit
sub connect {

    my $self = shift;
    my $telnet = $self->{'telnet'};

    # call our standard connect() method as usual first
    my $connected = $self->SUPER::connect();

    # detect failure to connect
    return $connected if (!$connected);

    # we need to receive the first two characters before we do anything else (like login..)
    # these are the line feed (0x0A) and escape (0x1B) chars
    $telnet->get();
    $telnet->get();

    return $connected;
}

sub get_alarms {

    my $self = shift;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};

    my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

    my @alarms;

    my $cmd_output = $self->cmd($cmd);
    return undef if (!defined $cmd_output);

    my @output_messages = $self->parse($cmd_output);
    my @responses = $self->get_responses(@output_messages);

    foreach my $response (@responses) {

	my $alarm;
	
	if ($response =~ /^"([^,]+),([^:]+):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),.*:(.*),.*"$/) {

	    $alarm->{'aid'} = $1;
	    $alarm->{'aid_type'} = $2;
	    $alarm->{'not_code'} = $3;
	    $alarm->{'cond_type'} = $4;
	    $alarm->{'serv_affect'} = $5;
	    $alarm->{'date'} = $6;
	    $alarm->{'time'} = $7;
	    $alarm->{'description'} = $8;
	    
	    # remove any escaped quotes from description
	    $alarm->{'description'} =~ s/\\"//g; #"

	    # split date into year month day (although they never seem to include year for some reason..)
	    my @date = split(/-/, $alarm->{'date'});

	    if (@date == 3) {
		
		($alarm->{'year'}, $alarm->{'month'}, $alarm->{'day'}) = @date;
	    }
	    
	    else {
		
		($alarm->{'month'}, $alarm->{'day'}) = @date;
	    }

	    $alarm->{'time'} =~ s/-/:/g;
	    
	    push(@alarms, $alarm);
	}

	else {

	    print STDERR "get_alarms: Error parsing alarm string: $response\n";
	}
    }
    
    return @alarms;
}

sub get_autonomous_output {

    my ($self, $type) = @_;

    my $ctag = $self->{'ctag'};
    my $tid = $self->{'tid'};
    
    my $cmd = "RTRV-AO:::$ctag;";

    # send the command
    $self->send($cmd);

    # recv the raw output from this command
    my $output = $self->recv($cmd, $ctag);

    # grab all the autonomous messages from the output
    my @messages = $self->parse_autonomous_messages($output);

    my @results;

    # go through each auto message
    foreach my $message (@messages) {

	# make sure its of the right message type
	next if ($message->{'verb'} !~ /^REPT $type/);
	
	# go through each response in this message
	my $responses = $message->{'responses'};
	
	foreach my $response (@$responses) {

	    # get rid of the quotation marks
	    $response =~ s/\"//g;

	    if ($response =~ /(([^:]*):)?(.*)/){

		my $aid = $2;

		my $rest_of_line = $3;
		
		my ($not_code, $cond_type, $serv_affect, $ocr_dat, $ocrtm) = split(',', $rest_of_line);
		
		next unless ($not_code eq 'CL' || $not_code eq 'CR' || 
			     $not_code eq 'MJ' || $not_code eq 'MN');

		my $msg;
		
		$msg->{'aid'}         = $aid;
		$msg->{'not_code'}    = $not_code;
		$msg->{'cond_type'}   = $cond_type;
		$msg->{'serv_affect'} = $serv_affect;	

	        # shove in date & time from response header
	        $msg->{'date'} = $message->{'date'};
	        $msg->{'time'} = $message->{'time'};
	 
	        my @date = split(/-/, $msg->{'date'});
	
		if (@date == 3) {
		    
		    ($msg->{'year'}, $msg->{'month'}, $msg->{'day'}) = @date;
		}
		
		else {
		    
		    ($msg->{'month'}, $msg->{'day'}) = @date;
		}
	    
		push(@results, $msg);
	    }
    
	    else {	
		print STDERR "get_autonomous_output: Error parsing response: $response\n";
	    }
	}
    }
    
    return @results;
}

1;
