#--------------------------------------------------------------------
#----- GRNOC TL1 Cisco ONS15327 Device Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: svn+ssh://svn.grnoc.iu.edu/grnoc/perl-lib/GRNOC-TL1/tags/1.3.10/lib/GRNOC/TL1/Device/Cisco/ONS15327.pm $
#----- $Id: ONS15327.pm 19099 2012-08-06 19:56:18Z mrmccrac $
#-----
#----- This module handles the functionality for the Cisco ONS 15327
#----- TL1 device.  It inherts from the standard
#----- GRNOC::TL1::Device module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Cisco::ONS15327;
use base "GRNOC::TL1::Device";

use Data::Dumper;
use strict;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;

  my (%args) = @_;

  # set default port used for ONS 15327 if one was not given
  $args{'port'} = 3083 if ($args{'port'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);

  return $self;
}

# override the recv() method to handle retarded 15327 output responses...
# instead of relying entirely on COMPLD, read until ; is hit on a single line
# after an instance of COMPLD has been received
sub recv {

  my ($self, $cmd, $ctag) = @_;

  my $telnet = $self->{'telnet'};
  my $prompt = $self->{'prompt'};
  my $debug = $self->{'debug'};

  my $buf;

  print STDERR "recv: ctag is $ctag\n" if ($debug);

  # continually read lines
  while (my $line = $telnet->getline()) {

    # ignore lines which is the command that was typed in
    next if ($line =~ /$cmd/);

    print STDERR "recv: Line is $line" if ($debug);

    $buf .= $line;

    # is this the final output message for this ctag?
    if ($line =~ /^\s*M\s+$ctag\s+(COMPLD|DENY)/) {

      # read until the next ; at beginning of line
      my $regex = "/^;\\s*/gm";
      print STDERR "Waiting for: $regex\n" if ($debug);
      my ($prematch, $match) = $telnet->waitfor($regex);
      print STDERR "Got: $prematch" if ($debug);
      print STDERR "Got: $match" if ($debug);

      $buf .= $prematch;
      return $buf;
    }
  }
}

sub get_alarms {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /.*"([^,]*),([^:]*):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]*),([^:]*):([^,]*),([^\"]*)"$/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'date'} = $6;
      $alarm->{'time'} = $7;
      $alarm->{'location'} = $8;
      $alarm->{'direction'} = $9;
      $alarm->{'description'} = $10;
      $alarm->{'description'} =~ s/\\"//g;
      $alarm->{'aid_det'} = $11;

      my @date = split(/-/, $alarm->{'date'});

      if (@date == 3) {

        $alarm->{'year'} = $date[0];
        $alarm->{'month'} = $date[1];
        $alarm->{'day'} = $date[2];
      }
      else {

        $alarm->{'month'} = $date[0];
        $alarm->{'day'} = $date[1];
      }

      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }

  return @alarms;
}

sub get_inventory {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-INV:$tid:ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^,]+),([^:]*)::.*"$/) {

      $device->{'aid'} = $1;
      $device->{'type'} = $2;

      # strip \ and "s
      $device->{'type'} =~ s/\\//g;
      $device->{'type'} =~ s/"//g;

      ($device->{'part_number'}) = $response =~ /PN=([^,]*),/;
      $device->{'part_number'} =~ s/\\//g;
      $device->{'part_number'} =~ s/"//g;

      ($device->{'version'}) = $response =~ /HWREV=([^,]*),/;
      $device->{'version'} =~ s/\\//g;
      $device->{'version'} =~ s/"//g;

      ($device->{'firmware'}) = $response =~ /FWREV=([^,]*),/;
      $device->{'firmware'} =~ s/\\//g;
      $device->{'firmware'} =~ s/"//g;

      ($device->{'serial'}) = $response =~ /SN=([^,]*),/;
      $device->{'serial'} =~ s/\\//g;
      $device->{'serial'} =~ s/"//g;

      ($device->{'clei'}) = $response =~ /CLEI=([^,]*),/;
      $device->{'clei'} =~ s/\\//g;
      $device->{'clei'} =~ s/"//g;

      ($device->{'product_id'}) = $response =~ /PID=([^,]*),/;
      $device->{'product_id'} =~ s/\s+//;
      $device->{'product_id'} =~ s/\\//g;
      $device->{'product_id'} =~ s/"//g;

      ($device->{'vendor_id'}) = $response =~ /VID=([^,]*),/;
      $device->{'vendor_id'} =~ s/\\//g;
      $device->{'vendor_id'} =~ s/"//g;

      push(@inventory, $device);
    }

    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  return @inventory;
}

sub get_facilities {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-FAC:$tid:ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @facilities;

  foreach my $response (@responses) {

    my $facility;

    if ($response =~ /^"([^:]+)::PAYLOAD=([^:]+):([^"]+)"$/) {

      $facility->{'aid'} = $1;
      $facility->{'payload'} = $2;
      $facility->{'state'} = $3;

      push(@facilities, $facility);
    }

    else {

      print STDERR "get_facilities: Error parsing response: $response\n";
    }
  }

  return @facilities;
}

sub get_facility {

  my ($self, $aid, $payload) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-$payload:$tid" . ":" . "$aid:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my $facility;

  my $response = $responses[0];

  if ($response =~ /^"([^:]+):.*:([^"]*)"$/) {

    $facility->{'aid'} = $1;
    $facility->{'state'} = $2;

    ($facility->{'description'}) = $response =~ /NAME=\\"([^\\"]*)\\"/;
    ($facility->{'tx_power'}) = $response =~ /OPT=([^,]*),/;
    ($facility->{'rx_power'}) = $response =~ /OPR=([^,]*),/;
  }

  else {

    print STDERR "get_facility: Error parsing response: $response\n";
  }

  return $facility;
}

sub get_oc3_interface {

  my ($self, $aid) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-OC3:$tid:$aid:$ctag;";

  return $self->_get_ocn_interface($cmd);
}

sub get_oc12_interface {

  my ($self, $aid) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-OC12:$tid:$aid:$ctag;";

  return $self->_get_ocn_interface($cmd);
}

sub get_oc48_interface {

  my ($self, $aid) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-OC48:$tid:$aid:$ctag;";

  return $self->_get_ocn_interface($cmd);
}

# private method that does the work for get_oc3, oc12, oc48, oc192_interface
sub _get_ocn_interface {

  my ($self, $cmd) = @_;

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my $response = $responses[0];

  my $oc;

  if ($response =~ /^"([^:]+):.*:([^,]*),.*"$/) {

    $oc->{'aid'} = $1;
    $oc->{'state'} = $2;

    ($oc->{'description'}) = $response =~ /NAME=\\"([^\\"]*)\\"/;
    ($oc->{'tx_power'}) = $response =~ /OPT=([^,]*),/;
    ($oc->{'rx_power'}) = $response =~ /OPR=([^,]*),/;
  }

  else {

    print STDERR "_get_ocn_interface: Error parsing response: $response\n";
  }

  return $oc;
}

sub get_cross_connects {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-CRS:$tid:ALL:$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @cross_connects;

  foreach my $response (@responses) {

    my $cross_connect;

    if ($response =~ /^"([^,]+),([^:]+):([^,]*),([^:]*):.*:([^,]*),.*"$/) {

      $cross_connect->{'from_aid'} = $1;
      $cross_connect->{'to_aid'} = $2;
      $cross_connect->{'type'} = $3;
      $cross_connect->{'rate'} = $4;
      $cross_connect->{'state'} = $5;

      ($cross_connect->{'description'}) = $response =~ /CKTID=\\"([^\\"]*)\\":/;

      push(@cross_connects, $cross_connect);
    }

    else {

      print STDERR "get_cross_connects: Error parsing response: $response\n";
    }
  }

  return @cross_connects;
}

sub get_time {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-TOD:$tid" . "::" . "$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  # only expecting one response
  my $response = $responses[0];

  my $time;

  # "2009,11,02,08,55,26,-360:CST"
  if ($response =~ /^"(\d+),(\d+),(\d+),(\d+),(\d+),(\d+),([^:]+):(.*)"$/) {

    $time->{'year'} = $1;
    $time->{'month'} = $2;
    $time->{'day'} = $3;
    $time->{'hour'} = $4;
    $time->{'min'} = $5;
    $time->{'sec'} = $6;
    $time->{'difference'} = $7;
    $time->{'timezone'} = $8;

    return $time;
  }

  else {

    print STDERR "get_time: Error parsing response: $response\n";

    return undef;
  }
}

sub backup_database {

  my ($self, $ftp_user, $ftp_pass, $ftp_host, $remote_filename) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "COPY-RFILE:$tid:RFILE-DB:$ctag" . "::TYPE=RFBU,DEST=\"FTP://$ftp_user:$ftp_pass\@$ftp_host/$remote_filename\";";

  # return true if backup command was successful (COMPLD)

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my ($output_message) = $self->parse($cmd_output);
  return 1
    if ($output_message->{'completion_code'} eq "COMPLD");
}

1;
