#--------------------------------------------------------------------
#----- GRNOC TL1 Nortel HDXc Device Module
#-----
#----- Copyright(C) 2008 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module handles the functionality for the Nortel HDXc TL1 device.  It
#----- inherts from the standard GRNOC::TL1::Device module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Nortel::HDXc;
use base "GRNOC::TL1::Device";

use Data::Dumper;
use Date::Calc qw(Monday_of_Week);
use strict;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;

  my (%args) = @_;

  # set default port used for HDXc if one was not given
  $args{'port'} = 23 if ($args{'port'} eq '');

  # set default prompt used for HDXc if one was not given
  $args{'prompt'} = 'TL1 Engine>' if ($args{'prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);

  return $self;
}

sub get_alarms {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /"([^,]+),([^:]+):([^,]+),([^,]+),([^,]+),([^,]+),([^,]+),([^,]*),([^:]+):\\\"([^\\]+)\\\",.*,\\\"([^\\]+)\\\".*:([^,]+),:YEAR=([^\"]+)"$/) {

      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'month_day'} = $6;
      $alarm->{'time'} = $7;
      $alarm->{'location'} = $8;
      $alarm->{'direction'} = $9;
      $alarm->{'description'} = $10;
      $alarm->{'aid_format'} = $11;
      $alarm->{'alarm_ids'} = $12;
      $alarm->{'year'} = $13;


      my @two = split(/-/, $alarm->{'month_day'});
      $alarm->{'month'} = $two[0];
      $alarm->{'day'} = $two[1];

      $alarm->{'time'} =~ s/-/:/g;

      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }

  return @alarms;
}

sub get_inventory {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-INVENTORY:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);

  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^:]*)::.*"$/) {

      $device->{'aid'} = $1;

      ($device->{'pec'}) = $response =~ /PEC=([^,]*),/;
      ($device->{'date'}) = $response =~ /,MDAT=([^,]*),/;
      ($device->{'type'}) = $response =~ /,TYPE=([^,]*),/;
      ($device->{'subtype'}) = $response =~ /,SUBTYPE=([^,]*),/;
      ($device->{'version'}) = $response =~ /,REL=([^,]*),/;
      ($device->{'serial'}) = $response =~ /,SER=([^"]*)"/;
      ($device->{'clei'}) = $response =~ /,CLEI=([^,]*),/;

      # skip over it if it is FILLER
      next if ($device->{'type'} eq "FILLER");

      $device->{'version'} =~ s/^\s//;
      $device->{'version'} =~ s/\s$//;

      # reformat date to be month-year if needed
      if ($device->{'date'} ne "") {

        my $year;
        my $week;
        my $month;
        my $day;

        ( $year, $week ) = $device->{'date'} =~ /([^wk]+)wk(.*)/;
        ( $year, $month, $day ) = Monday_of_Week( $week, $year );

        $device->{'date'} = $month . "-" . $year;
      }

      push(@inventory, $device);
    }

    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  # also do a RTRV-SHELF
  $cmd = "RTRV-SHELF:$tid" . "::$ctag;";

  $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  @output_messages = $self->parse($cmd_output);
  @responses = $self->get_responses(@output_messages);

  foreach my $response (@responses) {

    my $shelf;

    if ($response =~ /^"([^:]+):.*"$/) {

      $shelf->{'aid'} = $1;

      ($shelf->{'pec'}) = $response =~ /PEC=([^,]*),/;
      ($shelf->{'date'}) = $response =~ /,MDAT=([^,]*),/;
      ($shelf->{'type'}) = $response =~ /,TYPE=([^,]*),/;
      ($shelf->{'version'}) = $response =~ /,REL=([^,]*),/;
      ($shelf->{'serial'}) = $response =~ /,SER=([^,]*),/;

      # reformat date to be month-year if needed
      if ($shelf->{'date'} ne "") {

        my $year;
        my $week;
        my $month;
        my $day;

        ( $year, $week ) = $shelf->{'date'} =~ /([^wk]+)wk(.*)/;
        ( $year, $month, $day ) = Monday_of_Week( $week, $year );

        $shelf->{'date'} = $month . "-" . $year;
      }

      push(@inventory, $shelf);
    }

    else {

      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  return @inventory;
}

sub get_cross_connects {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-CRS-ALL:$tid" . "::$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @cross_connects;

  foreach my $response (@responses) {

    my $cross_connect;

    if ($response =~ /^"([^,]*),([^:]*):([^,]*),([^:]*):.*"$/) {

      $cross_connect->{'from_aid'} = $1;
      $cross_connect->{'to_aid'} = $2;
      $cross_connect->{'type'} = $3;
      $cross_connect->{'rate'} = $4;

      ($cross_connect->{'description'}) = $response =~ /LABEL=([^,|^:]*)/;
      $cross_connect->{'description'} =~ s/\\"//g;

      ($cross_connect->{'state'}) = $response =~ /AST=([^,|^:]*)/;
      ($cross_connect->{'id'}) = $response =~ /CONNID=([^,|^:]*)/;

      push(@cross_connects, $cross_connect);
    }

    else {

      print STDERR "get_cross_connects: Error parsing response: $response\n";
    }
  }

  return @cross_connects;
}

sub _get_ocn_interfaces {

  my ($self, $aid, $ocn) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-$ocn" . ":$tid" . "::" . "$ctag;";

  my $cmd_output = $self->cmd($cmd);
  return undef if (!defined $cmd_output);

  my @output_messages = $self->parse($cmd_output);
  my @responses = $self->get_responses(@output_messages);
  my @ocn_interfaces;

  foreach my $response (@responses) {

    my $interface;

    if ($response =~ /^"([^:]+):.*:([^"]+)"$/) {

      ($interface->{'aid'}, $interface->{'state'}) = ($1, $2);

      ($interface->{'description'}) = $response =~ /LABEL=\\"([^\\"]*)\\"/;

      push(@ocn_interfaces, $interface);
    }

    else {

      print STDERR "_get_ocn_interfaces: Error parsing response: $response\n";
    }
  }

  return @ocn_interfaces;
}

sub get_oc3_interfaces {

  my ($self, $aid) = @_;

  return $self->_get_ocn_interfaces($aid, "OC3");
}

sub get_oc12_interfaces {

  my ($self, $aid) = @_;

  return $self->_get_ocn_interfaces($aid, "OC12");
}

sub get_oc48_interfaces {

  my ($self, $aid) = @_;

  return $self->_get_ocn_interfaces($aid, "OC48");
}

sub get_oc192_interfaces {

  my ($self, $aid) = @_;

  return $self->_get_ocn_interfaces($aid, "OC192");
}

1;
