#--------------------------------------------------------------------
#----- GRNOC TL1 Parser Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module parses the desired raw input & output from a GRNOC::TL1 device.
#----- These include TL1 Input Messages and TL1 Output Messages.
#--------------------------------------------------------------------

package GRNOC::TL1::Parser;

use Data::Dumper;
use strict;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;
  my $self = {
	      debug => 0,
	      @_
	     };
  
  bless($self, $class);
  return $self;
}

sub parse_input_message {

  my ($self, $message) = @_;

  print STDERR "_parse_input_message: parsing $message\n" if ($self->{'debug'});

  # get rid of final ;
  $message =~ s/;$//;
  
  my ($command_code, $tid, $aid, $ctag, $general_block, $data_block) = split(/:/, $message);

  my $result;
  
  $result->{'command_code'} = $command_code;
  $result->{'tid'} = $tid;
  $result->{'aid'} = $aid;
  $result->{'ctag'} = $ctag;
  $result->{'general_block'} = $general_block;
  $result->{'data_block'} = $data_block;

  return $result;
}

sub parse_output_messages {

  my ($self, $output) = @_;

  # examine each line to determine state
  my @lines = split(/\n/, $output);

  # grab the output messages
  my @output_messages = $self->_find_output_messages(@lines);

  # return them
  return @output_messages;
}

sub _find_output_messages {

  my ($self, @lines) = @_;
  my @output_messages;
  
  my $i;

  for ($i = 0; $i < @lines; $i++) {

    print STDERR "_find_output_messages: Line is " . $lines[$i] . "\n" if ($self->{'debug'});
    
    # peek ahead see if its an output message response id
    if ($lines[$i + 1] =~ /^\s*M\s+([^\s]+)\s+(.*)/) {

      my $ctag = $1;
      my $completion_code = $2;
      
      # this line should be the response header
      my ($sid, $date, $time) = $lines[$i] =~ /^\s*([^\s]+)\s+([^\s]+)\s+(.*)/;
      
      # create an output message record to store data
      my $output_message;
      $output_message->{'sid'} = $sid;
      $output_message->{'date'} = $date;
      $output_message->{'time'} = $time;
      $output_message->{'ctag'} = $ctag;
      $output_message->{'completion_code'} = $completion_code;

      # get responses from response block lines
      my $responses;
      ($responses, $i) = &_get_output_message_responses($i + 2, @lines);
      $output_message->{'responses'} = $responses;

      # add this to our list of output messages
      push(@output_messages, $output_message);
    }
  }

  return @output_messages;
}

sub _get_output_message_responses {

  my ($index, @lines) = @_;
  my @responses;
  my $i;
  
  for ($i = $index; $i < @lines; $i++) { 
    
    my $line = $lines[$i];
    
    # we know we are done if we hit a ; at the beginning of a line
    last if ($line =~ /^;/);

    # strip initial whitespace
    $line =~ s/^\s*//;

    my $next_line = $lines[$i + 1];
    $next_line =~ s/^\s*//;

    # have we found the start of another output message?
    if ($next_line =~ /^M\s+/) {
      
      return (\@responses, $i - 1);
    }

    # have we found the start of an autonomous message?
    if ($next_line =~ /^A\s+/) {

      return (\@responses, $i + 1);
    }

    # skip lines that dont start and end with a "
    next if ($line !~ /^".*"\s*$/);

    # include this line as a response
    push(@responses, $line);
  }

  return (\@responses, $i);
}

sub parse_autonomous_messages {

  my ($self, $output) = @_;

  my @messages;

  my @lines = split(/\n/, $output);

  for (my $i = 0; $i < @lines; $i++) {

    # peek ahead to the next line to see if its the autonomous id
    # infineras have *, **, *C in addition to A for alarm codes :(
    if ($lines[$i + 1] =~ /^\s*(A|\*|\*\*|\*C)\s+([^\s]+)\s+(.*)/) {

      # parsed out the next line (auto id)
      my ($alarm_code, $atag, $verb) = ($1, $2, $3);

      # this current line should be the auto header
      my ($sid, $date, $time) = $lines[$i] =~ /^\s*([^\s]+)\s+([^\s]+)\s+(.*)/;

      my $autonomous_message;

      $autonomous_message->{'sid'} = $sid;
      $autonomous_message->{'date'} = $date;
      $autonomous_message->{'time'} = $time;
      $autonomous_message->{'alarm_code'} = $alarm_code;
      $autonomous_message->{'atag'} = $atag;
      $autonomous_message->{'verb'} = $verb;
      
      my @responses;

      # get auto block lines for this message
      for (my $j = $i + 2; $j < @lines; $j++) {

	my $line = $lines[$j];

	# we're done if we hit a line that doesn't start/end with " "
	if ($line !~ /^\s*".*"\s*$/) {

	  $i = $j - 1;
	  last;
	}
	
	# strip beginning & ending whitespace
	$line =~ s/^\s*//g;
	$line =~ s/\s*$//g;
	
	push(@responses, $line);
      }

      $autonomous_message->{'responses'} = \@responses;
      push(@messages, $autonomous_message);
    }
  }

  return @messages;
}

1;
